@@ -382,7 +382,7 @@ F2x_addshift(GEN x, GEN y, long d)
   GEN xd,yd,zd = (GEN)avma;
   long a,lz,ny = lgpol(y), nx = lgpol(x);
   long vs = x[1];
-
+  if (nx == 0) return y;
   x += 2; y += 2; a = ny-d;
   if (a <= 0)
   {
@@ -1025,7 +1025,7 @@ F2xq_trace(GEN x, GEN T)
 GEN
 F2xq_conjvec(GEN x, GEN T)
 {
-  long i, l = F2x_degree(T);
+  long i, l = 1+F2x_degree(T);
   GEN z = cgetg(l,t_COL);
   gel(z,1) = F2x_copy(x);
   for (i=2; i<l; i++) gel(z,i) = F2xq_sqr(gel(z,i-1), T);
