@@ -182,7 +182,7 @@ Fq_to_FF(GEN x, GEN ff)
   switch(ff[1])
   {
   case t_FF_FpXQ:
-    r= is_int ? scalarpol(x, varn(T)): x;
+    r= is_int ? scalarpol(x, varn(T)): ZX_copy(x);
     break;
   case t_FF_F2xq:
     r= is_int ? Z_to_F2x(x,T[1]): ZX_to_F2x(x);
@@ -896,12 +896,12 @@ long
 FF_ispower(GEN x, GEN K, GEN *pt)
 {
   ulong pp;
-  GEN r, T, p;
+  GEN z, r, T, p;
   pari_sp av = avma;
 
   if (FF_equal0(x)) { if (pt) *pt = gcopy(x); return 1; }
   _getFF(x, &T, &p, &pp);
-  if (pt) *pt = cgetg(5,t_FFELT);
+  z = pt? cgetg(5,t_FFELT): NULL;
   switch(x[1])
   {
   case t_FF_FpXQ:
@@ -915,7 +915,7 @@ FF_ispower(GEN x, GEN K, GEN *pt)
     break;
   }
   if (!r) { avma = av; return 0; }
-  if (pt) { (void)_mkFF(x,*pt,r); }
+  if (pt) { *pt = z; (void)_mkFF(x,z,r); }
   return 1;
 }
 
