@@ -800,6 +800,7 @@ idealsqrtn_int(GEN nf, GEN A, long n, GEN *pB)
   }
   /* compute valuations at ramified primes */
   root = ramified_root(nf, idealadd(nf, nf_get_diff(nf), A), A, n);
+  if (!root) return 0;
   /* remove ramified primes */
   if (isint1(root))
     root = matid(nf_get_degree(nf));
@@ -820,7 +821,8 @@ idealsqrtn_int(GEN nf, GEN A, long n, GEN *pB)
     if (!Z_ispowerall(a,n,&b)) return 0;
     J = idealadd(nf, b, A);
     A = idealdivexact(nf, idealpows(nf,J,n), A);
-    if (pB) root = odd(i)? idealdivexact(nf, root, J): idealmul(nf, root, J);
+    /* div and not divexact here */
+    if (pB) root = odd(i)? idealdiv(nf, root, J): idealmul(nf, root, J);
   }
   if (pB) *pB = root;
   return 1;
