@@ -1045,7 +1045,7 @@ primecertify(GEN nf, GEN beta, ulong p, GEN bad)
 
     /* cf rootsof1_Fl */
     g = pgener_Fl_local(q, L);
-    (void)u_lvalrem((q-1) / p, p, &m);
+    m = (q-1) / p;
     gg = utoipos( Fl_powu(g, m, q) ); /* order p in (Z/q)^* */
     og = mkmat2(mkcol(utoi(p)), mkcol(gen_1)); /* order of g */
 
@@ -1061,7 +1061,6 @@ primecertify(GEN nf, GEN beta, ulong p, GEN bad)
       {
         GEN t = nf_to_Fp_coprime(nf, gel(beta,j), modpr);
         t = utoipos( Fl_powu(t[2], m, q) );
-        /* FIXME: implement Fl_log_Shanks */
         C[j] = itou( Fp_log(t, gg, og, qq) ) % p;
       }
       r = lg(M);
