@@ -1070,6 +1070,11 @@ find_isogenous(GEN a4,GEN a6, ulong ell, struct meqn *MEQN, GEN g, GEN T,GEN p)
 {
   ulong pp = itou_or_0(p);
   long e = (pp && pp <= 2*ell+3) ? 2+factorial_lval(ell, pp): 1;
+  if (signe(a4)==0 || signe(a6)==0)
+  {
+    if (DEBUGLEVEL>0) err_printf("[%c: j=%ld]",MEQN->type,signe(a4)==0 ?0: 1728);
+    return NULL;
+  }
   if (e > 1)
   {
     GEN pe = powiu(p, e);
