@@ -769,8 +769,8 @@ lfunthetacheckinit(GEN data, GEN t, long m, long bitprec)
     if (bitprec > bitprecnew) goto INIT;
     get_cone(t, &rt, &alt);
     tdom = theta_get_tdom(thetainit);
-    r = rtodbl(gel(tdom,1));
-    al= rtodbl(gel(tdom,2)); if (rt >= r && alt <= al) return data;
+    r = gtodouble(gel(tdom,1));
+    al= gtodouble(gel(tdom,2)); if (rt >= r && alt <= al) return data;
   }
 INIT:
   return lfunthetainit_i(data, t, m, bitprec);
@@ -1364,7 +1364,7 @@ lfuninit(GEN lmisc, GEN dom, long der, long bitprec)
     if (isintzero(eno)) eno = NULL;
   }
   theta = lfun_init_theta(ldata, eno, &S);
-  if (eno && lg(ldata)==7)
+  if (eno && !r)
     R = gen_0;
   else
   {
@@ -1372,11 +1372,8 @@ lfuninit(GEN lmisc, GEN dom, long der, long bitprec)
     ldata = shallowcopy(ldata);
     gel(ldata, 6) = gel(v,3);
     r = gel(v,1);
-    if (isintzero(r))
-      setlg(ldata,7); /* no pole */
-    else
-      gel(ldata, 7) = r;
-    R = lfunrtoR(ldata, nbits2prec(S.D));
+    R = gel(v,2);
+    if (isintzero(r)) setlg(ldata,7); else gel(ldata, 7) = r;
   }
   h = divru(mplog2(S.precmax), S.m0);
   k = ldata_get_k(ldata);
@@ -1761,7 +1758,7 @@ lfunhardy(GEN lmisc, GEN t, long bitprec)
   prec = precision(argz);
   a = gsub(gmulsg(d, gmul(t, gmul2n(argz,-1))),
            gmul(expot,glog(gnorm(z),prec)));
-  h = lfunlambda_OK(linit, z, mkvec(t), bitprec);
+  h = lfunlambda_OK(linit, z, dom, bitprec);
   if (typ(ldata_get_dual(ldata))==t_INT)
     h = mulreal(h, w2);
   else
@@ -2017,10 +2014,11 @@ lfunrootres(GEN data, long bitprec)
   ldata = lfunmisc_to_ldata_shallow(data);
   r = ldata_get_residue(ldata);
   k = ldata_get_k(ldata);
+  w = ldata_get_rootno(ldata);
   if (r) r = normalize_simple_pole(r, stoi(k));
   if (!r || residues_known(r))
   {
-    w = lfunrootno(data, bitprec);
+    if (isintzero(w)) w = lfunrootno(data, bitprec);
     if (!r)
       r = R = gen_0;
     else
@@ -2032,7 +2030,6 @@ lfunrootres(GEN data, long bitprec)
   if (lg(r) > 2) pari_err_IMPL("multiple poles in lfunrootres");
   /* Now residue unknown, and r = [[be,0]]. */
   be = gmael(r, 1, 1);
-  w = ldata_get_rootno(ldata);
   if (ldata_isreal(ldata) && gequalm1(w))
     R = lfuntheta(linit, gen_1, 0, bitprec);
   else
