@@ -706,11 +706,14 @@ lllgramallgen(GEN x, long flag)
   return lll_finish(h,k-1,flag);
 }
 
+static int
+RgM_square(GEN x) { long l = lg(x); return l == 1 || l == lgcols(x); }
 static GEN
 lllallgen(GEN x, long flag)
 {
   pari_sp av = avma;
   if ((flag & LLL_GRAM) == 0) x = gram_matrix(x);
+  else if (!RgM_square(x)) pari_err_DIM("qflllgram");
   return gerepilecopy(av, lllgramallgen(x, flag));
 }
 GEN
@@ -726,6 +729,7 @@ static GEN
 lllall(GEN x, long flag)
 {
   pari_sp av = avma;
+  if ((flag & LLL_GRAM) && !RgM_square(x)) pari_err_DIM("qflllgram");
   return gerepilecopy(av, ZM_lll(x, LLLDFT, flag));
 }
 GEN
@@ -744,6 +748,7 @@ lllfp(GEN x, double D, long flag)
   pari_sp av = avma;
   GEN h;
   if (n <= 1) return lll_trivial(x,flag);
+  if ((flag & LLL_GRAM) && !RgM_square(x)) pari_err_DIM("qflllgram");
   h = ZM_lll(RgM_rescale_to_int(x), D, flag);
   return gerepilecopy(av, h);
 }
