@@ -237,13 +237,19 @@ RgX_cmbf(GEN p, long i, GEN BLOC, GEN Lmod, GEN Lfac, GEN *F)
 
 static GEN factor_domain(GEN x, GEN flag);
 
+static GEN
+ok_bloc(GEN f, GEN BLOC, ulong c)
+{
+  GEN F = poleval(f, BLOC);
+  return issquarefree(c ? gmul(F,mkintmodu(1,c)): F)? F: NULL;
+}
 static GEN
 RgXY_factor_squarefree(GEN f, GEN dom)
 {
   pari_sp av = avma;
   ulong i, c = itou_or_0(residual_characteristic(f));
   long vy = gvar2(f), val = RgX_valrem(f, &f), n = RgXY_degreex(f);
-  GEN Lmod, F = NULL, BLOC = NULL, Lfac = coltrunc_init(degpol(f)+2);
+  GEN y, Lmod, F = NULL, BLOC = NULL, Lfac = coltrunc_init(degpol(f)+2);
   if (val)
   {
     GEN x = pol_x(varn(f));
@@ -254,13 +260,19 @@ RgXY_factor_squarefree(GEN f, GEN dom)
     }
     vectrunc_append(Lfac, x); if (!degpol(f)) return Lfac;
   }
+  y = pol_x(vy);
   for(;;)
   {
     for (i = 0; !c || i < c; i++)
     {
-      BLOC = gpowgs(gaddgs(pol_x(vy), i), n+1);
-      F = poleval(f, BLOC);
-      if (issquarefree(c ? gmul(F,mkintmodu(1,c)): F)) break;
+      BLOC = gpowgs(gaddgs(y, i), n+1);
+      if ((F = ok_bloc(f, BLOC, c))) break;
+      if (c)
+      {
+        BLOC = random_FpX(n+1, vy, utoipos(c));
+        gel(BLOC,lg(BLOC)-1) = gen_1;
+        if ((F = ok_bloc(f, BLOC, c))) break;
+      }
     }
     if (!c || i < c) break;
     n++;
@@ -2099,8 +2111,7 @@ rescale_init(GEN c, int *exact, long *emin, GEN *D)
   {
     case t_REAL:
       *exact = 0;
-      if (!signe(c)) return;
-      e = expo(c) - bit_prec(c);
+      e = signe(c)? expo(c) - bit_prec(c): expo(c);
       break;
     case t_INT:
       if (!signe(c)) return;
@@ -2178,6 +2189,10 @@ Q_divmuli_to_int(GEN x, GEN d, GEN n)
       for (i=2; i<l; i++) gel(y,i) = Q_divmuli_to_int(gel(x,i), d,n);
       return y;
 
+    case t_RFRAC:
+      av = avma;
+      return gerepileupto(av, gmul(x,mkfrac(n,d)));
+
     case t_POLMOD:
       retmkpolmod(Q_divmuli_to_int(gel(x,2), d,n), RgX_copy(gel(x,1)));
   }
@@ -2207,6 +2222,9 @@ Q_divi_to_int(GEN x, GEN d)
       for (i=2; i<l; i++) gel(y,i) = Q_divi_to_int(gel(x,i), d);
       return y;
 
+    case t_RFRAC:
+      return gdiv(x,d);
+
     case t_POLMOD:
       retmkpolmod(Q_divi_to_int(gel(x,2), d), RgX_copy(gel(x,1)));
   }
