@@ -2083,6 +2083,7 @@ QXQ_inv(GEN A, GEN B)
   pari_sp av2, av = avma;
   forprime_t S;
   pari_timer ti;
+  int checked = 0;
   if (is_scalar_t(typ(A))) return scalarpol(ginv(A), varn(B));
   /* A a QX, B a ZX */
   A = Q_primitive_part(A, &D);
@@ -2098,8 +2099,15 @@ QXQ_inv(GEN A, GEN B)
     a = ZX_to_Flx(A, p);
     b = ZX_to_Flx(B, p);
     /* if p | Res(A/G, B/G), discard */
-    if (!Flx_extresultant(b,a,p, &Vp,&Up)) continue;
-
+    if (!Flx_extresultant(b,a,p, &Vp,&Up))
+    {
+      if (!checked)
+      {
+        if (degpol(ZX_gcd(A,B))!=0) pari_err_INV("QXQ_inv",mkpolmod(A,B));
+        checked = 1;
+      }
+      continue;
+    }
     if (!U)
     { /* First time */
       U = ZX_init_CRT(Up,p,varn(A));
