@@ -912,7 +912,7 @@ fft(GEN Omega, GEN p, GEN f, long step, long l)
     gel(ff,i+l3+1) = gsub(g02, g13);
   }
   ff = gerepilecopy(ltop,ff);
-  for (i=0; i<l; i++) f[i] = ff[i+1];
+  for (i=0; i<l; i++) gel(f,i) = gel(ff,i+1);
 }
 
 GEN
@@ -1060,8 +1060,8 @@ dft(GEN p, long k, long NN, long Lmax, long bit, GEN F, GEN H, long polreal)
   C = cgetg(Lmax+1,t_VEC); C++;
   pc = cgetg(Lmax+1,t_VEC); pc++;
   pd = cgetg(Lmax+1,t_VEC); pd++;
-  pc[0] = q[2];  for (i=n+1; i<Lmax; i++) gel(pc,i) = gen_0;
-  pd[0] = qd[2]; for (i=n;   i<Lmax; i++) gel(pd,i) = gen_0;
+  gel(pc,0) = gel(q,2);  for (i=n+1; i<Lmax; i++) gel(pc,i) = gen_0;
+  gel(pd,0) = gel(qd,2); for (i=n;   i<Lmax; i++) gel(pd,i) = gen_0;
 
   ltop = avma;
   W = cgetg(k+1,t_VEC);
@@ -2069,14 +2069,14 @@ tocomplex(GEN x, long l, long bit)
   return y;
 }
 
-/* x,y are t_COMPLEX of t_REALs or t_REAL, compare lexicographically,
- * up to 2^-e absolute error */
+/* x,y are t_COMPLEX of t_REALs or t_REAL, compare wrt |Im x| - |Im y|,
+ * then Re x - Re y, up to 2^-e absolute error */
 static int
 cmp_complex_appr(void *E, GEN x, GEN y)
 {
   long e = (long)E;
   GEN z, xi, yi, xr, yr;
-  long sxi, syi;
+  long sz, sxi, syi;
   if (typ(x) == t_COMPLEX) { xr = gel(x,1); xi = gel(x,2); sxi = signe(xi); }
   else { xr = x; xi = NULL; sxi = 0; }
   if (typ(y) == t_COMPLEX) { yr = gel(y,1); yi = gel(y,2); syi = signe(yi); }
@@ -2094,15 +2094,13 @@ cmp_complex_appr(void *E, GEN x, GEN y)
   }
   else
   {
-    long sz;
-    z = addrr_sign(xi, 1, yi, -1);
-    sz = signe(z);
+    z = addrr_sign(xi, 1, yi, -1); sz = signe(z);
     if (sz && expo(z) >= e) return (int)sz;
   }
   /* |Im x| ~ |Im y|, sort according to real parts */
-  z = subrr(xr, yr);
-  if (expo(z) >= e) return (int)signe(z);
-  /* Re x ~ Re y. Place negative absolute value before positive */
+  z = subrr(xr, yr); sz = signe(z);
+  if (sz && expo(z) >= e) return (int)sz;
+  /* Re x ~ Re y. Place negative imaginary part before positive */
   return (int) (sxi - syi);
 }
 
