@@ -263,7 +263,13 @@ pari_mt_init(void)
     MPI_Comm_size(MPI_COMM_WORLD, &pari_MPI_size);
     MPI_Comm_rank(MPI_COMM_WORLD, &pari_MPI_rank);
     if (pari_MPI_rank) pari_MPI_child();
-    if (!pari_mt_nbthreads) pari_mt_nbthreads = pari_MPI_size-1;
+#ifdef _IOFBF
+  /* HACK: most MPI implementation does not handle stdin well.
+  stdinsize is sufficient for the largest test file to fit */
+  setvbuf(stdin,pari_malloc(128*1024),_IOFBF,128*1024);
+#endif
+    if (!pari_mt_nbthreads)
+      pari_mt_nbthreads = maxss(1, pari_MPI_size-1);
   }
   else
   {
